
##
## Usage : ./Sample_nc2geotiff.py
##
## Tutorial Page
##   https://www.eorc.jaxa.jp/JASMES/tutorial/tutorial_002.html
##   https://www.eorc.jaxa.jp/JASMES/tutorial/tutorial_002_j.html
##
## License:MIT
## Copyright(c) 2023 JAXA/EORC
##

import os
import sys
import struct
import numpy as np
import netCDF4
from osgeo import gdal, osr

argvs = sys.argv

print("### START ###")

# input netCDF (GCOM-C/SGLI or VIIRS)
INPUT = argvs[1]

# output GeoTiFF
OUTPUT = argvs[2]

# prod name
PRD = argvs[3]

# read netCDF4
nc = netCDF4.Dataset(INPUT, 'r')
nc.set_auto_maskandscale(False)

### latitude, longitude
#   SGLI STD, VIIRS
dlat = nc.dimensions['latitude'].size
dlon = nc.dimensions['longitude'].size

#   SGLI NRT
#dlat = nc.dimensions['Latitude'].size
#dlon = nc.dimensions['Longitude'].size

data = nc.variables[PRD]

### max, slope, offset
#   SGLI STD, VIIRS
valid_max = nc.variables[PRD].valid_max
slope     = nc.variables[PRD].scale_factor
offset    = nc.variables[PRD].add_offset

#   SGLI NRT
#valid_max = nc.variables[PRD].Maximum_valid_DN
#slope     = nc.variables[PRD].scale_factor
#offset    = nc.variables[PRD].add_offset

### grid_interval
#   SGLI STD, VIIRS
grid_interval = nc.grid_interval

#   SGLI NRT
#grid_interval = 0.0025

### upper_left
#   SGLI STD, VIIRS (pixel center -> top left corner of the pixel)
upper_left_longitude = nc.upper_left_longitude - (grid_interval * 0.5)
upper_left_latitude  = nc.upper_left_latitude + (grid_interval * 0.5)

#   SGLI NRT
#upper_left_longitude = nc.Upper_left_longitude
#upper_left_latitude  = nc.Upper_left_latitude


# pixel size
dx = grid_interval
dy = grid_interval*-1

# value = DN * slope + offset
val = data*slope+offset
otemp      = np.zeros((dlat,dlon),dtype=np.float32)
otemp[:,:] = val[:,:]
otemp[data>valid_max] = np.nan
otemp = otemp.astype(np.float32)

# GeoTiFF
driver = gdal.GetDriverByName('GTiff')

# outputfile, cols, rows, band num, data type
cols      = otemp.shape[1]
rows      = otemp.shape[0]
outRaster = driver.Create(OUTPUT, cols, rows, 1, gdal.GDT_Float32)

# upper_left_longitude, pixel size(x), rotate, upper_left_latitude, rotate, pixel size(y)
outRaster.SetGeoTransform((upper_left_longitude, dx, 0, upper_left_latitude, 0, dy))

# band
outband = outRaster.GetRasterBand(1)
outband.WriteArray(otemp)

# EPSG
outRasterSRS = osr.SpatialReference()
outRasterSRS.ImportFromEPSG(4326)
outRaster.SetProjection(outRasterSRS.ExportToWkt())

outband.FlushCache()
outRaster = None

print("### END output : "+OUTPUT+" ###")
sys.exit(0)
